﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using Telerik.Windows.Controls.Map;

namespace DragDropPinPoints
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        // "is initialized" flag
        private bool isInitialized;

        // "is dragging started" flag
        private bool isDraggingStarted;

        // the shift from location point of PinPoint to clicked point
        private Point shiftPoint;

        // current dragged element
        private FrameworkElement draggedElement;

        // start mouse location
        private Location startLocation;


        public MainWindow()
        {
            InitializeComponent();
        }

        private void RadMap1_InitializeCompleted(object sender, EventArgs e)
        {
            if (!this.isInitialized)
            {
                this.isInitialized = true;

                // attach radMap events
                this.RadMap1.MouseMove += this.OnMouseMove;
                this.RadMap1.MouseLeftButtonUp += this.OnMouseLeftButtonUp;

                // initializing draagable objects
                MapPinPoint pinPoint = new MapPinPoint();
                pinPoint.Width = 25;
                pinPoint.Height = 25;
                pinPoint.Background = new SolidColorBrush(Colors.Blue);

                MapLayer.SetLocation(pinPoint, new Location(37.747440943384, -95.2481111547551));
                this.MakeElementDraggable(pinPoint);

                this.informationLayer.Items.Add(pinPoint);

                Pushpin pushPin = new Pushpin();
                MapLayer.SetLocation(pushPin, new Location(35, -93));
                this.MakeElementDraggable(pushPin);

                this.informationLayer.Items.Add(pushPin);
            }
        }

        private void MakeElementDraggable(FrameworkElement element)
        {
            // attach mouse events
            MapShape shape = element as MapShape;
            if (shape != null)
            {
                shape.MouseLeftButtonDown += this.OnMouseLeftButtonDown;
            }
            else
            {
                element.MouseLeftButtonDown += this.OnMouseLeftButtonDown;
            }
        }

        private void OnMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            FrameworkElement element = sender as FrameworkElement;
            if (element != null)
            {
                this.RadMap1.CaptureMouse();

                this.draggedElement = element;

                Point point = e.GetPosition(this.RadMap1);

                // get the shift from location point of element to clicked point
                Location location = MapLayer.GetLocation(element);
                this.startLocation = Location.GetCoordinates(this.RadMap1, point);

                this.shiftPoint = location.GetPoint(this.RadMap1);
                this.shiftPoint.X -= point.X;
                this.shiftPoint.Y -= point.Y;

                // start dragging
                this.isDraggingStarted = true;

                // prevent propagating event to the map
                e.Handled = true;
            }
        }

        private void OnMouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (this.isDraggingStarted && this.draggedElement != null)
            {
                this.RadMap1.ReleaseMouseCapture();

                this.isDraggingStarted = false;
                this.draggedElement = null;
            }
        }

        private void OnMouseMove(object sender, MouseEventArgs e)
        {
            if (this.isDraggingStarted)
            {
                FrameworkElement element = this.draggedElement as FrameworkElement;
                if (element != null)
                {
                    // get the current point
                    Point point = e.GetPosition(this.RadMap1);
                    point.X += this.shiftPoint.X;
                    point.Y += this.shiftPoint.Y;

                    // get the location of current point
                    Location newLocation = Location.GetCoordinates(this.RadMap1, point);

                    // set the location to element
                    MapLayer.SetLocation(element, newLocation);
                }
            }
        }
    }
}
